"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_1 = __importDefault(require("mocha"));
const chai_1 = require("chai");
const mocha_2 = require("mocha");
const ramda_1 = __importDefault(require("ramda"));
class TestUtils {
    constructor() {
        this.TC_DEFAULT = { type: '' };
        this.DEFAULT_REPORTER = { reporter: 'my-reporter.js' };
    }
    createSuite(ctx, opts, suitename) {
        // @ts-ignore
        ctx.mocha = new mocha_1.default();
        // @ts-ignore
        ctx.suite = mocha_2.Suite.create(ctx.mocha.suite, suitename);
        ctx.suite['enableTimeouts'](false);
        return Promise.resolve(ctx.suite);
    }
    createTestCase(ctx, opts, testname, fn) {
        return __awaiter(this, void 0, void 0, function* () {
            let options = ramda_1.default.merge(this.TC_DEFAULT, opts);
            if (!ctx.mocha)
                yield this.createSuite(ctx, {}, '');
            let t = new mocha_2.Test(testname, fn);
            // @ts-ignore
            ctx.suite.addTest(t);
            return Promise.resolve(t);
        });
    }
    parseChaiExpect(expected, actual, ...chains) {
        chains = ramda_1.default.flatten(chains);
        let res = chai_1.expect(expected);
        let lastChain = chains.pop();
        chains.forEach(chain => res = res[chain]);
        if (actual)
            return res[lastChain](actual);
        else
            return res[lastChain];
    }
    // DEFAULT_REPORTER = {reporter:'mochawesome'};
    runTest(ctx, opts) {
        return new Promise((resolve, reject) => {
            let options = ramda_1.default.merge(this.DEFAULT_REPORTER, opts);
            ctx.mocha.reporter(options.reporter, {
                reportTitle: options.reportTitle,
                reportFilename: options.reportFilename,
                showPassed: options.showPassed,
                overwrite: options.overwrite,
                autoOpen: options.autoOpen,
                quiet: true
            });
            ctx.runner = ctx.mocha.run(() => resolve(ctx.runner));
        });
    }
}
exports.TestUtils = TestUtils;
